/*
Speckle - Calcul de la déformation de l'image d'un objet ponctuel par la turbulence atmosphérique.

Copyright (C) 2012 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

/** Algorithme de simulation pour l'environnement SimuApplet.
    Déformation d'une image par la turbulence atmosphérique.
*/


createArray = function(length) {
    var a, args, i;
    
    a = new Array(length || 0);
    
    if (arguments.length > 1) {
        args = Array.prototype.slice.call(arguments, 1);
        for (i = 0; i < length; i++) {
            a[i] = createArray.apply(this, args);
        }
    } else {
        for (i = 0; i < length; i++)
            a[i] = 0;
    }
    
    return a;
}

function isCanvasSupported() {
    var elem = document.createElement('canvas');
    return !!(elem.getContext && elem.getContext('2d'));
}

function versionApplet() {
    var applet = document.createElement('applet');
    var width, height; // espace disponible pour l'applet
    if (typeof(window.innerWidth) == 'number') {
        // autre que IE
        width = window.innerWidth;
        height = window.innerHeight;
    } else if ( document.documentElement && (document.documentElement.clientWidth || document.documentElement.clientHeight) ) {
        // IE 6+ en mode 'standards compliant'
        width = document.documentElement.clientWidth;
        height = document.documentElement.clientHeight;
    } else {
        width = '100%';
        height = '100%';
    }
    applet.setAttribute('width', width);
    applet.setAttribute('height', height);
    applet.setAttribute('archive', 'Speckle.jar,SimuLab.jar');
    applet.setAttribute('code', 'Speckle');

    document.body.appendChild(applet);
}

if (isCanvasSupported()) {
    new Speckle();
} else {
    // on ne peut pas utiliser excanvas ou flashcanvas pour manipuler des bitmaps offscreen
    // donc on utilise la version sous forme d'applet en alternative pour les vieux navigateurs
    if (window.addEventListener)
        window.addEventListener('load', function() { versionApplet(); }, false);
    else if (window.attachEvent)
        window.attachEvent('onload', function() { versionApplet(); });
}

function Speckle() {
    this.tmasque = null; // double[][]
    this.dx = 256;
    this.dy = 128;
    this.dz = 0;
    this.z0 = null; // double[][]
    this.w = 0.;
    this.champ = null; // double[][]
    
    this.mz = 0;         // nombre de Zernikes
    this.zi = null;      // tableau pour mzZernikes (double[][][])
    this.ph = null;      // phase instantanée (double[][])
    this.z = null;       // coefficients des Zernikes (double[])
    this.tps = null;     // enveloppe stat. de Kolmogorov (double[])
    this.maxph = 0.;     // maximum de la phase (pour affichage)
    this.maxi = 0.;      // maximum de l'intensité (pour affichage)
    this.pupille = null; // pupille unité (double[][])
    
    this.applet = new SimuApplet(this);
    this.applet.appletDoc("speckle_doc.xml");
}

Speckle.prototype.initCalculLive = function(lin) {
    var z2, z3, z4, z5, z6, z7etPlus, som, zn, amp, table_temp, table_nulle, toggle, doverR, i, o, j;
    
    // on récupère le nb de Zernikes
    this.mz = lin.lireEntier('nb-zernike');
    // on récupère les infos on/off sur les Zernikes
    z2 = lin.lireString('z2');
    z3 = lin.lireString('z3');
    z4 = lin.lireString('z4');
    z5 = lin.lireString('z5');
    z6 = lin.lireString('z6');
    z7etPlus = lin.lireString('z7etplus');
    doverR = lin.lireDouble('doverr');

    this.ph = createArray(this.dx/2, this.dy);
    if ( this.mz <= 1 )
        this.mz = 1;   // au moins un Zernike
    this.z = createArray(this.mz);
    this.zi = createArray(this.dx/2, this.dy, this.mz);
    this.tps = createArray(this.mz);
    som = 0.;
    zn = [ 0 , 0 ];
    table_temp = createArray(this.dx/2, this.dy);
    table_nulle = createArray(this.dx/2, this.dy);
    toggle = createArray(this.mz);
    
    // on allume ou éteint les Zernikes
    for ( i = 0 ; i < this.mz ; i++ ) {
        toggle[i] = 1.;      // on initialise en allumant tout
    }        
    
    // on éteint selon ce qui a été coché
    if ( this.mz > 0 && z2 == 'off' ) toggle[0] = 0.;
    if ( this.mz > 1 && z3 == 'off' ) toggle[1] = 0.;
    if ( this.mz > 2 && z4 == 'off' ) toggle[2] = 0.;
    if ( this.mz > 3 && z5 == 'off' ) toggle[3] = 0.;
    if ( this.mz > 4 && z6 == 'off' ) toggle[4] = 0.;
    if ( this.mz > 5 && z7etPlus == 'off' ) {
        for ( i = 5 ; i < this.mz ; i++ ) {
            toggle[i] = 0;      
        }        
    }
    
    // récupration d'une pupille unité
    this.pupille = Zernike.calcZernike(1, this.dx/2);

    // calcul et stockage de mz zernikes avec spectre de K en -11/3 en variance et -11/6 en amplitude
    for ( o=0 ; o <= this.mz-1 ; o++ ) {
        zn = Zernike.zernumero(o + 2);

        table_temp = Zernike.calcZernike(o+2, this.dx/2);

        for (i=0; i<this.dx/2; i++) {
            for(j=0; j<this.dy; j++) {
                this.zi[i][j][o] = table_temp[i][j];
            }
        }

        this.tps[o]    = toggle[o] * 1.0 / Math.pow(zn[0], (11.0 / 6.0));
        som       = som + this.tps[o] * this.tps[o];
    }

    // normalisation pour doverR
    amp = Math.sqrt(1.03*Math.pow(doverR, (5.0/3.0)) / som);
    for ( i = 0 ; i < this.mz ; i++ ) {
        this.tps[i] = this.tps[i] * amp;
    }

    // calcul des parametres de display optimaux
    // si tous les zernikes sont a 1, que vaut la phase ?
    this.maxph = 0.;
    for ( i = 0 ; i < this.mz ; i++ ) {
        this.maxph = this.maxph + this.tps[i];
    }

    table_temp = Zernike.calcZernike(1, this.dx/2);
    //Zernike.complexToComplex2D(1,dx/2,table_temp,table_nulle);
    BibliMath.FFTcmplx2cmplx2D(1, this.dx/2, table_temp, table_nulle);

    for (i=0; i<this.dx/2; i++) {
        for (j=0; j<this.dy; j++) {
            this.maxi = Math.max(this.maxi, (table_temp[i][j]*table_temp[i][j]));
        }
    }
    this.maxi = this.maxi * 10./ doverR / doverR;

    this.champ = createArray(this.dx, this.dy);
}

Speckle.prototype.calculLive = function() {
    var out, ph, eiphiRe, eiphiIm, ims, i, o, j;
    
    out = new ListeValeurs();
    ph = createArray(this.dx/2, this.dy);
    eiphiRe = createArray(this.dx/2, this.dy);
    eiphiIm = createArray(this.dx/2, this.dy);
    ims = createArray(this.dx/2, this.dy);

    // on génère l'intensité des modes
    for ( i = 0 ; i < this.mz ; i++ ) {
        this.z[i] = this.tps[i] * 2. * (Math.random()-0.5);
    }

    // on crée la phase instantanée
    for ( o=0 ; o <= this.mz-1 ; o++ ) {
        for (i=0; i<this.dx/2; i++) {
            for (j=0; j<this.dy; j++) {
                ph[i][j] = ph[i][j] + this.z[o] * this.zi[i][j][o];
            }
        }
    }

    // on crée l'amplitude complexe, avec un signe - un point sur 2 afin d'avoir un résultat de TF centré
    for (i=0; i<this.dx/2; i++) {
        for (j=0; j<this.dy; j++) {
            eiphiRe[i][j] = Math.pow(-1.,i) * Math.pow(-1.,j) * Math.cos(ph[i][j]) * this.pupille[i][j];
            eiphiIm[i][j] = Math.pow(-1.,i) * Math.pow(-1.,j) * Math.sin(ph[i][j]) * this.pupille[i][j];
        }
    }

    // on calcule la TF de l'amplitude complexe
    //Zernike.complexToComplex2D(1,dx/2,eiphiRe,eiphiIm);
    BibliMath.FFTcmplx2cmplx2D(1, this.dx/2, eiphiRe, eiphiIm);

    // on calcule l'intensité
    for (i=0; i<this.dx/2; i++) {
        for (j=0; j<this.dy; j++) {
            ims[i][j] = eiphiRe[i][j] * eiphiRe[i][j] + eiphiIm[i][j] * eiphiIm[i][j];
        }
    }

    for (i=0; i<this.dx; i++) {
        for (j=0; j<this.dy; j++) {
            if ( i < this.dx/2 ) 
                this.champ[i][j] = ph[i][j];        // la partie gauche de l'image est occupée par la phase
            else
                this.champ[i][j] = ims[i-this.dx/2][j];  // la partie droite de l'image est occupée par l'intensité
        }
    }

    for (i=0; i<this.dx; i++) {
        for (j=0; j<this.dy; j++) {
            if ( i < this.dx/2 )
                this.champ[i][j] = 255.*this.champ[i][j] / this.maxph + 128;
            else
                this.champ[i][j] = 255.*this.champ[i][j] / this.maxi;
            if ( this.champ[i][j] > 255 )
                this.champ[i][j] = 255;
            if ( this.champ[i][j] < 0 )
                this.champ[i][j] = 0;
        }
    }
    
    out.ajouter('champ', this.champ);
    
    return out;
}
