/*
Zernike - algorithme de calcul de polynômes de Zernike

Copyright (C) 2012 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

Zernike = function() {

return {

    createArray: function(length) {
        var a, args, i;
        
        a = new Array(length || 0);
        
        if (arguments.length > 1) {
            args = Array.prototype.slice.call(arguments, 1);
            for (i = 0; i < length; i++) {
                a[i] = Zernike.createArray.apply(this, args);
            }
        } else {
            for (i = 0; i < length; i++)
                a[i] = 0;
        }
        
        return a;
    },
    
    /** Calcule le Zernike normalise (somme(z^2) = 1) de rang zn.
    Convention ici : Zernike(1) = piston, Zernike(2) = tilt, etc...
    */
    //public static double[][] calcZernike (int zn, int dimension) {
    calcZernike: function(zn, dimension) {
        var z, tempo, r, znm, n, m, i, j, k;
        
        z = this.createArray(dimension, dimension);
        znm = this.zernumero(zn);
        n = znm[0];
        m = znm[1];
        
        if ( (zn % 2) == 1 ) {  // zn impair
            for ( i=0 ; i <= (n-m)/2 ; i++ ) {
                tempo = Math.pow(-1.,i) * Math.exp(BibliMath.alngam(n-i+1)
                    -(BibliMath.alngam(i+1) + BibliMath.alngam((n+m)/2.-i+1.) + BibliMath.alngam((n-m)/2.-i+1.)));
                for ( j=0 ; j < dimension ; j++ ) {
                    for ( k=0 ; k < dimension ; k++ ) {
                        r = Math.sqrt((j-dimension/2.)*(j-dimension/2.)+(k-dimension/2.)*(k-dimension/2.)) / dimension * 4.;
                        if ( r <= 1. ) {
                            z[j][k]=z[j][k] + Math.pow(r,(n-2.*i))*tempo;
                        } else {
                            z[j][k] = 0.;    // en dehors d'un disque de rayon unité (pupille) le Zernike est nul
                        }
                    }
                }
            }  
            if ( m == 0 ) {
                for ( j=0 ; j < dimension ; j++ ) {
                    for ( k=0 ; k < dimension ; k++ ) {
                        z[j][k] = z[j][k] * Math.sqrt(n+1.); 
                    }
                }
            }
            else {
                for ( j=0 ; j < dimension ; j++ ) {
                    for ( k=0 ; k < dimension ; k++ ) {
                        z[j][k] = z[j][k]*Math.sqrt(2.*(n+1.))*Math.sin(m*Math.atan2((k-dimension/2.),(j-dimension/2.)));
                    }
                }
            }
        }
        else {   // zn pair
            for ( i=0 ; i <= (n-m)/2 ; i++ ) {
                tempo = Math.pow(-1.,i) * Math.exp(BibliMath.alngam(n-i+1)
                    -(BibliMath.alngam(i+1) + BibliMath.alngam((n+m)/2.-i+1.) + BibliMath.alngam((n-m)/2.-i+1.)));
                for ( j=0 ; j < dimension ; j++ ) {
                    for ( k=0 ; k < dimension ; k++ ) {
                        r = Math.sqrt((j-dimension/2.)*(j-dimension/2.)+(k-dimension/2.)*(k-dimension/2.))/dimension*4.;
                        if ( r <= 1. ) {
                            z[j][k] = z[j][k]+Math.pow(r,(n-2.*i))*tempo;
                        }
                        else {
                            z[j][k] = 0.;    // en dehors d'un disque de rayon unité (pupille) le Zernike est nul
                        }
                    }
                }
            }
            if ( m == 0 ) {
                for ( j=0 ; j < dimension ; j++ ) {
                    for ( k=0 ; k < dimension ; k++ ) {
                        z[j][k] = z[j][k]*Math.sqrt(n+1.); 
                    }
                }
            }
            else {
                for ( j=0 ; j < dimension ; j++ ) {
                    for ( k=0 ; k < dimension ; k++ ) {
                        z[j][k] = z[j][k]*Math.sqrt(2.*(n+1.))*Math.cos(m*Math.atan2((k-dimension/2.),(j-dimension/2.)));
                    }
                }
            }
        }

        return z;

    },

    /** Calcul de l'ordre radial et azimutal d'un Zernike donné.
    Retourne un vecteur à deux valeurs : ordre radial, puis azimutal
    */
    //public static int[] zernumero(int zn) {
    zernumero: function(zn) {
        var j, resu, n, m;
        
        j = 0;
        resu = [ 0 , 0 ];

        for ( n=0 ; n <=100 ; n++ ) {
            for ( m=0 ; m <= n ; m++ ) {
                if ( (n-m) % 2 == 0 ) { // n-m pair
                    j = j+1;
                    if ( j == zn ) {
                        resu[0] = n;
                        resu[1] = m;
                        break;
                    }
                    if ( m != 0 ) {
                        j = j+1;
                        if ( j == zn ) {
                            resu[0] = n;
                            resu[1] = m;
                            break;
                        }
                    }
                } 
            }
        }
        return resu;
    }


};

}();
